@@ -61,7 +61,8 @@ struct _ClutterBackendPrivate
   guint double_click_distance;
 
   gdouble resolution;
-  gdouble units_per_em;
+
+  gfloat units_per_em;
 
   cairo_font_options_t *font_options;
 
@@ -84,7 +85,7 @@ clutter_backend_dispose (GObject *gobject)
   ClutterBackendPrivate *priv = CLUTTER_BACKEND (gobject)->priv;
   ClutterMainContext *clutter_context;
 
-  clutter_context = clutter_context_get_default ();
+  clutter_context = _clutter_context_get_default ();
 
   if (clutter_context && clutter_context->events_queue)
     {
@@ -102,55 +103,74 @@ clutter_backend_dispose (GObject *gobject)
   G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
 }
 
-static inline void
-update_units_per_em (ClutterBackend *backend)
+static gfloat
+get_units_per_em (ClutterBackend       *backend,
+                  PangoFontDescription *font_desc)
 {
-  ClutterBackendPrivate *priv = backend->priv;
-  const gchar *font_name;
+  gfloat units_per_em = -1.0;
+  gboolean free_font_desc = FALSE;
   gdouble dpi;
 
-  font_name = clutter_backend_get_font_name (backend);
   dpi = clutter_backend_get_resolution (backend);
 
-  if (G_LIKELY (font_name != NULL && *font_name != '\0'))
+  if (font_desc == NULL)
     {
-      PangoFontDescription *font_desc;
-      gdouble font_size = 0;
+      const gchar *font_name = clutter_backend_get_font_name (backend);
 
-      font_desc = pango_font_description_from_string (font_name);
-      if (G_LIKELY (font_desc != NULL))
+      if (G_LIKELY (font_name != NULL && *font_name != '\0'))
         {
-          gint pango_size;
-          gboolean is_absolute;
-
-          pango_size = pango_font_description_get_size (font_desc);
-          is_absolute =
-            pango_font_description_get_size_is_absolute (font_desc);
-          if (!is_absolute)
-            font_size = ((gdouble) font_size) / PANGO_SCALE;
-
-          pango_font_description_free (font_desc);
+          font_desc = pango_font_description_from_string (font_name);
+          free_font_desc = TRUE;
         }
+    }
 
-      /* 10 points at 96 DPI is 12 pixels */
-      priv->units_per_em = 1.2 * font_size
-                         * dpi
-                         / 96.0;
+  if (font_desc != NULL)
+    {
+      gdouble font_size = 0;
+      gint pango_size;
+      gboolean is_absolute;
+
+      pango_size = pango_font_description_get_size (font_desc);
+      is_absolute = pango_font_description_get_size_is_absolute (font_desc);
+
+      /* "absolute" means "device units" (usually, pixels); otherwise,
+       * it means logical units (points)
+       */
+      if (is_absolute)
+        font_size = (gdouble) pango_size / PANGO_SCALE;
+      else
+        font_size = (gdouble) pango_size / PANGO_SCALE
+                  * dpi
+                  / 96.0f;
+
+      /* 10 points at 96 DPI is 13.3 pixels */
+      units_per_em = (1.2f * font_size)
+                   * dpi
+                   / 96.0f;
     }
   else
-    priv->units_per_em = -1.0;
+    units_per_em = -1.0f;
+
+  if (free_font_desc)
+    pango_font_description_free (font_desc);
+
+  return units_per_em;
 }
 
 static void
 clutter_backend_real_resolution_changed (ClutterBackend *backend)
 {
-  update_units_per_em (backend);
+  backend->priv->units_per_em = get_units_per_em (backend, NULL);
+
+  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", backend->priv->units_per_em);
 }
 
 static void
 clutter_backend_real_font_changed (ClutterBackend *backend)
 {
-  update_units_per_em (backend);
+  backend->priv->units_per_em = get_units_per_em (backend, NULL);
+
+  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", backend->priv->units_per_em);
 }
 
 static void
@@ -243,17 +263,14 @@ _clutter_backend_create_stage (ClutterBackend  *backend,
                                ClutterStage    *wrapper,
                                GError         **error)
 {
-  ClutterMainContext  *context;
   ClutterBackendClass *klass;
+  ClutterStageManager *stage_manager;
   ClutterActor        *stage = NULL;
 
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), FALSE);
   g_return_val_if_fail (CLUTTER_IS_STAGE (wrapper), FALSE);
 
-  context = clutter_context_get_default ();
-
-  if (!context->stage_manager)
-    context->stage_manager = clutter_stage_manager_get_default ();
+  stage_manager = clutter_stage_manager_get_default ();
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (klass->create_stage)
@@ -264,7 +281,7 @@ _clutter_backend_create_stage (ClutterBackend  *backend,
 
   g_assert (CLUTTER_IS_STAGE_WINDOW (stage));
   _clutter_stage_set_window (wrapper, CLUTTER_STAGE_WINDOW (stage));
-  _clutter_stage_manager_add_stage (context->stage_manager, wrapper);
+  _clutter_stage_manager_add_stage (stage_manager, wrapper);
 
   return stage;
 }
@@ -280,36 +297,63 @@ _clutter_backend_redraw (ClutterBackend *backend,
     klass->redraw (backend, stage);
 }
 
+gboolean
+_clutter_backend_create_context (ClutterBackend  *backend,
+                                 gboolean         is_offscreen,
+                                 GError         **error)
+{
+  ClutterBackendClass *klass;
+
+  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), FALSE);
+
+  klass = CLUTTER_BACKEND_GET_CLASS (backend);
+  if (klass->create_context)
+    return klass->create_context (backend, is_offscreen, error);
+
+  return TRUE;
+}
+
 void
 _clutter_backend_ensure_context (ClutterBackend *backend,
                                  ClutterStage   *stage)
 {
-  ClutterBackendClass *klass;
   static ClutterStage *current_context_stage = NULL;
+  ClutterBackendClass *klass;
 
   g_return_if_fail (CLUTTER_IS_BACKEND (backend));
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
   if (current_context_stage != stage || !CLUTTER_ACTOR_IS_REALIZED (stage))
     {
+      ClutterStage *new_stage = NULL;
+
       if (!CLUTTER_ACTOR_IS_REALIZED (stage))
         {
-          CLUTTER_NOTE (MULTISTAGE, "Stage is not realized, unsetting");
-          stage = NULL;
+          new_stage = NULL;
+
+          CLUTTER_NOTE (MULTISTAGE,
+                        "Stage [%p] is not realized, unsetting the stage",
+                        stage);
         }
       else
-        CLUTTER_NOTE (MULTISTAGE, "Setting the new stage [%p]", stage);
-      
+        {
+          new_stage = stage;
+
+          CLUTTER_NOTE (MULTISTAGE,
+                        "Setting the new stage [%p]",
+                        new_stage);
+        }
+
       klass = CLUTTER_BACKEND_GET_CLASS (backend);
       if (G_LIKELY (klass->ensure_context))
-        klass->ensure_context (backend, stage);
+        klass->ensure_context (backend, new_stage);
       
       /* FIXME: With a NULL stage and thus no active context it may make more
        * sense to clean the context but then re call with the default stage 
        * so at least there is some kind of context in place (as to avoid
        * potential issue of GL calls with no context)
        */
-      current_context_stage = stage;
+      current_context_stage = new_stage;
 
       /* if the new stage has a different size than the previous one
        * we need to update the viewport; we do it by simply setting the
@@ -347,7 +391,7 @@ _clutter_backend_init_events (ClutterBackend *backend)
   ClutterBackendClass *klass;
   ClutterMainContext  *clutter_context;
 
-  clutter_context = clutter_context_get_default ();
+  clutter_context = _clutter_context_get_default ();
 
   g_return_if_fail (CLUTTER_IS_BACKEND (backend));
   g_return_if_fail (clutter_context != NULL);
@@ -359,8 +403,9 @@ _clutter_backend_init_events (ClutterBackend *backend)
     klass->init_events (backend);
 }
 
-ClutterUnit
-_clutter_backend_get_units_per_em (ClutterBackend *backend)
+gfloat
+_clutter_backend_get_units_per_em (ClutterBackend       *backend,
+                                   PangoFontDescription *font_desc)
 {
   ClutterBackendPrivate *priv;
 
@@ -368,8 +413,12 @@ _clutter_backend_get_units_per_em (ClutterBackend *backend)
 
   priv = backend->priv;
 
-  if (G_UNLIKELY (priv->units_per_em < 0))
-    update_units_per_em (backend);
+  /* recompute for the font description, but do not cache the result */
+  if (font_desc != NULL)
+    return get_units_per_em (backend, font_desc);
+
+  if (priv->units_per_em < 0)
+    priv->units_per_em = get_units_per_em (backend, NULL);
 
   return priv->units_per_em;
 }
@@ -391,7 +440,7 @@ clutter_get_default_backend (void)
 {
   ClutterMainContext *clutter_context;
 
-  clutter_context = clutter_context_get_default ();
+  clutter_context = _clutter_context_get_default ();
 
   return clutter_context->backend;
 }
@@ -602,6 +651,8 @@ clutter_backend_get_font_options (ClutterBackend *backend)
   cairo_font_options_set_antialias (priv->font_options,
                                     CAIRO_ANTIALIAS_DEFAULT);
 
+  g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
+
   return priv->font_options;
 }
 
@@ -661,7 +712,12 @@ clutter_backend_get_font_name (ClutterBackend *backend)
   if (G_LIKELY (priv->font_name))
     return priv->font_name;
 
+  /* if we have never been called then we need to set the
+   * default font and update everything that relies on the
+   * ::font-changed signal
+   */
   priv->font_name = g_strdup (DEFAULT_FONT_NAME);
+  g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
 
   return priv->font_name;
 }
