@@ -18,11 +18,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
+#error "Only <clutter/clutter.h> can be included directly."
+#endif
+
 #ifndef __CLUTTER_BEHAVIOUR_ROTATE_H__
 #define __CLUTTER_BEHAVIOUR_ROTATE_H__
 
@@ -43,41 +45,50 @@ typedef struct _ClutterBehaviourRotate          ClutterBehaviourRotate;
 typedef struct _ClutterBehaviourRotatePrivate   ClutterBehaviourRotatePrivate;
 typedef struct _ClutterBehaviourRotateClass     ClutterBehaviourRotateClass;
 
+/**
+ * ClutterBehaviourRotate:
+ *
+ * The #ClutterBehaviourRotate struct contains only private data and
+ * should be accessed using the provided API
+ *
+ * Since: 0.4
+ */
 struct _ClutterBehaviourRotate
 {
+  /*< private >*/
   ClutterBehaviour parent_instance;
 
-  /*< private >*/
   ClutterBehaviourRotatePrivate *priv;
 };
 
+/**
+ * ClutterBehaviourRotateClass:
+ *
+ * The #ClutterBehaviourRotateClass struct contains only private data
+ *
+ * Since: 0.4
+ */
 struct _ClutterBehaviourRotateClass
 {
+  /*< private >*/
   ClutterBehaviourClass parent_class;
 };
 
 GType clutter_behaviour_rotate_get_type (void) G_GNUC_CONST;
 
-ClutterBehaviour *clutter_behaviour_rotate_new  (ClutterAlpha           *alpha,
-                                                 ClutterRotateAxis       axis,
-                                                 ClutterRotateDirection  direction,
-                                                 gdouble                 angle_start,
-                                                 gdouble                 angle_end);
-ClutterBehaviour *clutter_behaviour_rotate_newx (ClutterAlpha           *alpha,
-                                                 ClutterRotateAxis       axis,
-                                                 ClutterRotateDirection  direction,
-                                                 ClutterFixed            angle_start,
-                                                 ClutterFixed            angle_end);
-
+ClutterBehaviour *     clutter_behaviour_rotate_new           (ClutterAlpha           *alpha,
+                                                               ClutterRotateAxis       axis,
+                                                               ClutterRotateDirection  direction,
+                                                               gdouble                 angle_start,
+                                                               gdouble                 angle_end);
 void                   clutter_behaviour_rotate_get_center    (ClutterBehaviourRotate *rotate,
 							       gint                   *x,
 							       gint                   *y,
 							       gint                   *z);
 void                   clutter_behaviour_rotate_set_center    (ClutterBehaviourRotate *rotate,
-							       gint                   x,
-							       gint                   y,
-							       gint                   z);
-
+							       gint                    x,
+							       gint                    y,
+							       gint                    z);
 ClutterRotateAxis      clutter_behaviour_rotate_get_axis      (ClutterBehaviourRotate *rotate);
 void                   clutter_behaviour_rotate_set_axis      (ClutterBehaviourRotate *rotate,
                                                                ClutterRotateAxis       axis);
@@ -90,12 +101,6 @@ void                   clutter_behaviour_rotate_get_bounds    (ClutterBehaviourR
 void                   clutter_behaviour_rotate_set_bounds    (ClutterBehaviourRotate *rotate,
                                                                gdouble                 angle_start,
                                                                gdouble                 angle_end);
-void                   clutter_behaviour_rotate_get_boundsx   (ClutterBehaviourRotate *rotate,
-                                                               ClutterFixed           *angle_start,
-                                                               ClutterFixed           *angle_end);
-void                   clutter_behaviour_rotate_set_boundsx   (ClutterBehaviourRotate *rotate,
-                                                               ClutterFixed            angle_start,
-                                                               ClutterFixed            angle_end);
 
 G_END_DECLS
 
