@@ -206,9 +206,46 @@ clutter_marshal_UINT__VOID (GClosure     *closure,
 
 /* VOID:BOXED (./clutter-marshal.list:5) */
 
-/* VOID:INT (./clutter-marshal.list:6) */
+/* VOID:BOXED,FLAGS (./clutter-marshal.list:6) */
+void
+clutter_marshal_VOID__BOXED_FLAGS (GClosure     *closure,
+                                   GValue       *return_value G_GNUC_UNUSED,
+                                   guint         n_param_values,
+                                   const GValue *param_values,
+                                   gpointer      invocation_hint G_GNUC_UNUSED,
+                                   gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__BOXED_FLAGS) (gpointer     data1,
+                                                  gpointer     arg_1,
+                                                  guint        arg_2,
+                                                  gpointer     data2);
+  register GMarshalFunc_VOID__BOXED_FLAGS callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
 
-/* VOID:INT64,INT64,FLOAT,BOOLEAN (./clutter-marshal.list:7) */
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__BOXED_FLAGS) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_boxed (param_values + 1),
+            g_marshal_value_peek_flags (param_values + 2),
+            data2);
+}
+
+/* VOID:INT (./clutter-marshal.list:7) */
+
+/* VOID:INT64,INT64,FLOAT,BOOLEAN (./clutter-marshal.list:8) */
 void
 clutter_marshal_VOID__INT64_INT64_FLOAT_BOOLEAN (GClosure     *closure,
                                                  GValue       *return_value G_GNUC_UNUSED,
@@ -249,7 +286,7 @@ clutter_marshal_VOID__INT64_INT64_FLOAT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT (./clutter-marshal.list:8) */
+/* VOID:INT,INT (./clutter-marshal.list:9) */
 void
 clutter_marshal_VOID__INT_INT (GClosure     *closure,
                                GValue       *return_value G_GNUC_UNUSED,
@@ -286,7 +323,44 @@ clutter_marshal_VOID__INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT,INT,INT (./clutter-marshal.list:9) */
+/* VOID:FLOAT,FLOAT (./clutter-marshal.list:10) */
+void
+clutter_marshal_VOID__FLOAT_FLOAT (GClosure     *closure,
+                                   GValue       *return_value G_GNUC_UNUSED,
+                                   guint         n_param_values,
+                                   const GValue *param_values,
+                                   gpointer      invocation_hint G_GNUC_UNUSED,
+                                   gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__FLOAT_FLOAT) (gpointer     data1,
+                                                  gfloat       arg_1,
+                                                  gfloat       arg_2,
+                                                  gpointer     data2);
+  register GMarshalFunc_VOID__FLOAT_FLOAT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__FLOAT_FLOAT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_float (param_values + 1),
+            g_marshal_value_peek_float (param_values + 2),
+            data2);
+}
+
+/* VOID:INT,INT,INT,INT (./clutter-marshal.list:11) */
 void
 clutter_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
                                        GValue       *return_value G_GNUC_UNUSED,
@@ -327,9 +401,9 @@ clutter_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT (./clutter-marshal.list:10) */
+/* VOID:OBJECT (./clutter-marshal.list:12) */
 
-/* VOID:OBJECT,OBJECT,PARAM (./clutter-marshal.list:11) */
+/* VOID:OBJECT,OBJECT,PARAM (./clutter-marshal.list:13) */
 void
 clutter_marshal_VOID__OBJECT_OBJECT_PARAM (GClosure     *closure,
                                            GValue       *return_value G_GNUC_UNUSED,
@@ -368,7 +442,7 @@ clutter_marshal_VOID__OBJECT_OBJECT_PARAM (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,POINTER (./clutter-marshal.list:12) */
+/* VOID:OBJECT,POINTER (./clutter-marshal.list:14) */
 void
 clutter_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
                                       GValue       *return_value G_GNUC_UNUSED,
@@ -405,7 +479,7 @@ clutter_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,BOOLEAN,BOOLEAN (./clutter-marshal.list:13) */
+/* VOID:STRING,BOOLEAN,BOOLEAN (./clutter-marshal.list:15) */
 void
 clutter_marshal_VOID__STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value G_GNUC_UNUSED,
@@ -444,7 +518,7 @@ clutter_marshal_VOID__STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (./clutter-marshal.list:14) */
+/* VOID:STRING,INT (./clutter-marshal.list:16) */
 void
 clutter_marshal_VOID__STRING_INT (GClosure     *closure,
                                   GValue       *return_value G_GNUC_UNUSED,
@@ -481,7 +555,7 @@ clutter_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT (./clutter-marshal.list:15) */
+/* VOID:UINT (./clutter-marshal.list:17) */
 
-/* VOID:VOID (./clutter-marshal.list:16) */
+/* VOID:VOID (./clutter-marshal.list:18) */
 
