@@ -68,6 +68,7 @@ struct _ClutterMasterClock
   GSource *source;
 
   guint updated_stages : 1;
+  guint ensure_next_iteration : 1;
 };
 
 struct _ClutterMasterClockClass
@@ -125,6 +126,12 @@ master_clock_is_running (ClutterMasterClock *master_clock)
         _clutter_stage_needs_update (l->data))
       return TRUE;
 
+  if (master_clock->ensure_next_iteration)
+    {
+      master_clock->ensure_next_iteration = FALSE;
+      return TRUE;
+    }
+
   return FALSE;
 }
 
@@ -152,6 +159,7 @@ master_clock_next_frame_delay (ClutterMasterClock *master_clock)
       /* When we have sync-to-vblank, we count on that to throttle
        * our frame rate, and otherwise draw frames as fast as possible.
        */
+      CLUTTER_NOTE (SCHEDULER, "vblank available and updated stages");
       return 0;
     }
 
@@ -160,24 +168,33 @@ master_clock_next_frame_delay (ClutterMasterClock *master_clock)
       /* If we weren't previously running, then draw the next frame
        * immediately
        */
+      CLUTTER_NOTE (SCHEDULER, "draw the first frame immediately");
       return 0;
     }
 
-  /* Otherwise, wait at least 1/frame_rate seconds since we last started a frame */
-
+  /* Otherwise, wait at least 1/frame_rate seconds since we last
+   * started a frame
+   */
   g_source_get_current_time (master_clock->source, &now);
 
   next = master_clock->prev_tick;
-  g_time_val_add (&next, 1000000 / clutter_get_default_frame_rate ());
+  g_time_val_add (&next, 1000000L / (gulong) clutter_get_default_frame_rate ());
 
   if (next.tv_sec < now.tv_sec ||
-      (next.tv_sec == now.tv_sec && next.tv_usec < now.tv_usec))
+      (next.tv_sec == now.tv_sec && next.tv_usec <= now.tv_usec))
     {
+      CLUTTER_NOTE (SCHEDULER, "Less than %lu microsecs",
+                    1000000L / (gulong) clutter_get_default_frame_rate ());
+
       return 0;
     }
   else
     {
-      return ((next.tv_sec - now.tv_sec) * 1000 +
+      CLUTTER_NOTE (SCHEDULER, "Waiting %lu msecs",
+                    (next.tv_sec - now.tv_sec) * 1000 +
+                    (next.tv_usec - now.tv_usec) / 1000);
+
+      return ((next.tv_sec  - now.tv_sec)  * 1000 +
               (next.tv_usec - now.tv_usec) / 1000);
     }
 }
@@ -212,7 +229,9 @@ clutter_clock_prepare (GSource *source,
   ClutterMasterClock *master_clock = clock_source->master_clock;
   int delay;
 
+  clutter_threads_enter ();
   delay = master_clock_next_frame_delay (master_clock);
+  clutter_threads_leave ();
 
   *timeout = delay;
 
@@ -226,7 +245,9 @@ clutter_clock_check (GSource *source)
   ClutterMasterClock *master_clock = clock_source->master_clock;
   int delay;
 
+  clutter_threads_enter ();
   delay = master_clock_next_frame_delay (master_clock);
+  clutter_threads_leave ();
 
   return delay == 0;
 }
@@ -243,6 +264,8 @@ clutter_clock_dispatch (GSource     *source,
 
   CLUTTER_NOTE (SCHEDULER, "Master clock [tick]");
 
+  clutter_threads_enter ();
+
   /* Get the time to use for this frame.
    */
   g_source_get_current_time (source, &master_clock->cur_tick);
@@ -274,6 +297,8 @@ clutter_clock_dispatch (GSource     *source,
 
   master_clock->prev_tick = master_clock->cur_tick;
 
+  clutter_threads_leave ();
+
   return TRUE;
 }
 
@@ -304,6 +329,7 @@ clutter_master_clock_init (ClutterMasterClock *self)
   self->source = source;
 
   self->updated_stages = TRUE;
+  self->ensure_next_iteration = FALSE;
 
   g_source_set_priority (source, CLUTTER_PRIORITY_REDRAW);
   g_source_set_can_recurse (source, FALSE);
@@ -410,3 +436,18 @@ _clutter_master_clock_advance (ClutterMasterClock *master_clock)
       clutter_timeline_do_tick (l->data, &master_clock->cur_tick);
     }
 }
+
+/**
+ * _clutter_master_clock_ensure_next_iteration:
+ * @master_clock: a #ClutterMasterClock
+ *
+ * Ensures that the master clock will run at least one iteration
+ */
+void
+_clutter_master_clock_ensure_next_iteration (ClutterMasterClock *master_clock)
+{
+  g_return_if_fail (CLUTTER_IS_MASTER_CLOCK (master_clock));
+
+  master_clock->ensure_next_iteration = TRUE;
+}
+
