@@ -1,11 +1,9 @@
 /*
- * Clutter COGL
+ * Cogl
  *
- * A basic GL/GLES Abstraction/Utility Layer
+ * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Authored By Matthew Allum  <mallum@openedhand.com>
- *
- * Copyright (C) 2008 OpenedHand
+ * Copyright (C) 2008,2009 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -65,7 +63,7 @@
 
 static void _cogl_program_free (CoglProgram *program);
 
-COGL_HANDLE_DEFINE (Program, program, program_handles);
+COGL_HANDLE_DEFINE (Program, program);
 
 static void
 _cogl_program_free (CoglProgram *program)
@@ -83,11 +81,8 @@ cogl_create_program (void)
   _COGL_GET_CONTEXT (ctx, 0);
 
   program = g_slice_new (CoglProgram);
-  program->ref_count = 1;
   program->gl_handle = glCreateProgramObjectARB ();
 
-  COGL_HANDLE_DEBUG_NEW (program, program);
-
   return _cogl_program_handle_new (program);
 }
 
@@ -99,7 +94,7 @@ cogl_program_attach_shader (CoglHandle program_handle,
   CoglShader *shader;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  
+
   if (!cogl_is_program (program_handle) || !cogl_is_shader (shader_handle))
     return;
 
@@ -114,7 +109,7 @@ cogl_program_link (CoglHandle handle)
 {
   CoglProgram *program;
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  
+
   if (!cogl_is_program (handle))
     return;
 
@@ -129,10 +124,15 @@ cogl_program_use (CoglHandle handle)
   CoglProgram *program;
   GLhandleARB gl_handle;
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  
+
   if (handle != COGL_INVALID_HANDLE && !cogl_is_program (handle))
     return;
 
+  /* The Cogl journal doesn't currently cope with the use of
+   * shaders so we have to flush all priitives whenever the
+   * current shader changes... */
+  _cogl_journal_flush ();
+
   if (handle == COGL_INVALID_HANDLE)
     gl_handle = 0;
   else
@@ -144,13 +144,13 @@ cogl_program_use (CoglHandle handle)
   glUseProgramObjectARB (gl_handle);
 }
 
-COGLint
+int
 cogl_program_get_uniform_location (CoglHandle   handle,
                                    const gchar *uniform_name)
 {
   CoglProgram *program;
   _COGL_GET_CONTEXT (ctx, 0);
-  
+
   if (!cogl_is_program (handle))
     return 0;
 
@@ -160,7 +160,7 @@ cogl_program_get_uniform_location (CoglHandle   handle,
 }
 
 void
-cogl_program_uniform_1f (COGLint uniform_no,
+cogl_program_uniform_1f (int uniform_no,
                          gfloat  value)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -168,7 +168,7 @@ cogl_program_uniform_1f (COGLint uniform_no,
 }
 
 void
-cogl_program_uniform_1i (COGLint uniform_no,
+cogl_program_uniform_1i (int uniform_no,
                          gint    value)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -176,7 +176,7 @@ cogl_program_uniform_1i (COGLint uniform_no,
 }
 
 void
-cogl_program_uniform_float (COGLint  uniform_no,
+cogl_program_uniform_float (int  uniform_no,
                             gint     size,
                             gint     count,
                             const GLfloat *value)
@@ -203,10 +203,10 @@ cogl_program_uniform_float (COGLint  uniform_no,
 }
 
 void
-cogl_program_uniform_int (COGLint  uniform_no,
+cogl_program_uniform_int (int  uniform_no,
                           gint     size,
                           gint     count,
-                          const COGLint *value)
+                          const int *value)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -230,7 +230,7 @@ cogl_program_uniform_int (COGLint  uniform_no,
 }
 
 void
-cogl_program_uniform_matrix (COGLint   uniform_no,
+cogl_program_uniform_matrix (int   uniform_no,
                              gint      size,
                              gint      count,
                              gboolean  transpose,
