@@ -14,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,20 +26,17 @@
 #include <glib/gi18n-lib.h>
 
 #include <string.h>
-
-#include <unistd.h>
-
-#include <sys/ioctl.h>
-#include <sys/stat.h>
 #include <sys/types.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
+#include <sys/stat.h>
+#include <sys/ioctl.h>
 #include <fcntl.h>
-
 #include <errno.h>
 
 #include "clutter-backend-x11.h"
-#include "clutter-device-manager-x11.h"
-#include "clutter-input-device-x11.h"
 #include "clutter-stage-x11.h"
 #include "clutter-x11.h"
 
@@ -49,16 +46,31 @@
 #include <X11/extensions/XInput.h>
 #endif
 
-#include "cogl/cogl.h"
-
-#include "../clutter-debug.h"
-#include "../clutter-device-manager.h"
 #include "../clutter-event.h"
 #include "../clutter-main.h"
+#include "../clutter-debug.h"
 #include "../clutter-private.h"
 
+#include "cogl/cogl.h"
+
 G_DEFINE_TYPE (ClutterBackendX11, clutter_backend_x11, CLUTTER_TYPE_BACKEND);
 
+struct _ClutterX11XInputDevice
+{
+  ClutterInputDevice device;
+
+#ifdef HAVE_XINPUT
+  XDevice           *xdevice;
+  XEventClass        xevent_list[5];   /* MAX 5 event types */
+  int                num_events;
+#endif
+};
+
+#ifdef HAVE_XINPUT
+void  _clutter_x11_register_xinput ();
+#endif
+
+
 /* atoms; remember to add the code that assigns the atom value to
  * the member of the ClutterBackendX11 structure if you add an
  * atom name here. do not change the order!
@@ -85,7 +97,6 @@ static ClutterBackendX11 *backend_singleton = NULL;
 /* various flags corresponding to pre init setup calls */
 static gboolean _no_xevent_retrieval = FALSE;
 static gboolean clutter_enable_xinput = FALSE;
-static gboolean clutter_enable_argb = FALSE;
 static Display  *_foreign_dpy = NULL;
 
 /* options */
@@ -113,13 +124,6 @@ clutter_backend_x11_pre_parse (ClutterBackend  *backend,
       env_string = NULL;
     }
 
-  env_string = g_getenv ("CLUTTER_DISABLE_ARGB_VISUAL");
-  if (env_string)
-    {
-      clutter_enable_argb = FALSE;
-      env_string = NULL;
-    }
-
   return TRUE;
 }
 
@@ -187,12 +191,9 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
 
       clutter_backend_set_resolution (backend, dpi);
 
-      /* register input devices */
-      backend_x11->device_manager =
-        g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_X11,
-                      "use-xinput-1", clutter_enable_xinput,
-                      "backend", backend_x11,
-                      NULL);
+#ifdef HAVE_XINPUT
+      _clutter_x11_register_xinput ();
+#endif
 
       if (clutter_synchronise)
         XSynchronize (backend_x11->xdpy, True);
@@ -340,49 +341,21 @@ clutter_backend_x11_get_features (ClutterBackend *backend)
   return CLUTTER_FEATURE_STAGE_USER_RESIZE | CLUTTER_FEATURE_STAGE_CURSOR;
 }
 
-gboolean
-clutter_backend_x11_handle_event (ClutterBackendX11 *backend_x11,
-                                  XEvent *xevent)
-{
-  return FALSE;
-}
-
-static ClutterDeviceManager *
-clutter_backend_x11_get_device_manager (ClutterBackend *backend)
-{
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
-
-  if (G_UNLIKELY (backend_x11->device_manager == NULL))
-    {
-      backend_x11->device_manager =
-        g_object_new (CLUTTER_TYPE_DEVICE_MANAGER_X11,
-                      "use-xinput-1", clutter_enable_xinput,
-                      "backend", backend_x11,
-                      NULL);
-    }
-
-  return backend_x11->device_manager;
-}
-
 static void
 clutter_backend_x11_class_init (ClutterBackendX11Class *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterBackendClass *backend_class = CLUTTER_BACKEND_CLASS (klass);
-  ClutterBackendX11Class *backendx11_class = CLUTTER_BACKEND_X11_CLASS (klass);
 
   gobject_class->constructor = clutter_backend_x11_constructor;
   gobject_class->dispose = clutter_backend_x11_dispose;
   gobject_class->finalize = clutter_backend_x11_finalize;
 
-  backend_class->pre_parse = clutter_backend_x11_pre_parse;
-  backend_class->post_parse = clutter_backend_x11_post_parse;
-  backend_class->init_events = clutter_backend_x11_init_events;
-  backend_class->add_options = clutter_backend_x11_add_options;
-  backend_class->get_features = clutter_backend_x11_get_features;
-  backend_class->get_device_manager = clutter_backend_x11_get_device_manager;
-
-  backendx11_class->handle_event = clutter_backend_x11_handle_event;
+  backend_class->pre_parse        = clutter_backend_x11_pre_parse;
+  backend_class->post_parse       = clutter_backend_x11_post_parse;
+  backend_class->init_events      = clutter_backend_x11_init_events;
+  backend_class->add_options      = clutter_backend_x11_add_options;
+  backend_class->get_features     = clutter_backend_x11_get_features;
 }
 
 static void
@@ -490,13 +463,11 @@ clutter_x11_set_display (Display *xdpy)
  * clutter_x11_enable_xinput:
  *
  * Enables the use of the XInput extension if present on connected
- * XServer and support built into Clutter. XInput allows for multiple
- * pointing devices to be used.
+ * XServer and support built into Clutter.  XInput allows for multiple
+ * pointing devices to be used. This must be called before
+ * clutter_init().
  *
- * This function must be called before clutter_init().
- *
- * Since XInput might not be supported by the X server, you might
- * want to use clutter_x11_has_xinput() to see if support was enabled.
+ * You should use #clutter_x11_has_xinput to see if support was enabled.
  *
  * Since: 0.8
  */
@@ -666,28 +637,238 @@ clutter_x11_remove_filter (ClutterX11FilterFunc func,
 }
 
 void
-_clutter_x11_select_events (Window xwin)
+_clutter_x11_register_xinput ()
 {
 #ifdef HAVE_XINPUT
-  ClutterDeviceManager *manager;
-  const GSList *l;
+  XDeviceInfo *xdevices = NULL;
+  XDeviceInfo *info = NULL;
 
-  if (G_UNLIKELY (backend_singleton == NULL))
+  XDevice *xdevice = NULL;
+
+  XInputClassInfo *xclass_info = NULL;
+
+  gint opcode, event, error;
+  gint res;
+  gint num_devices = 0;
+  gint num_events = 0;
+  gint i = 0, j = 0;
+  gboolean have_an_xpointer = FALSE;
+
+  ClutterBackendX11      *x11b;
+  ClutterX11XInputDevice *device = NULL;
+
+  ClutterMainContext  *context;
+
+  GSList *input_devices = NULL;
+
+  if (!backend_singleton)
     {
       g_critical ("X11 backend has not been initialised");
+      return;
+    }
+
+  if (!clutter_enable_xinput)
+    {
+      CLUTTER_NOTE (BACKEND, "Not enabling XInput");
+      return;
+    }
+
+  context = _clutter_context_get_default ();
+
+  backend_singleton->have_xinput = FALSE;
+
+  /* is the XInput extension available? */
+  res = XQueryExtension (backend_singleton->xdpy, "XInputExtension",
+                         &opcode, &event,
+                         &error);
+  if (!res)
+    {
+      CLUTTER_NOTE (BACKEND, "X Input extension not available");
+      return;
+    }
+
+  x11b = backend_singleton;
+
+  xdevices = XListInputDevices (x11b->xdpy, &num_devices);
+
+  CLUTTER_NOTE (BACKEND, "%d XINPUT devices found", num_devices);
+
+  if (num_devices == 0)
+    return;
+
+  for (i = 0; i < num_devices; i++)
+    {
+      info = xdevices + i;
+      num_events = 0;
+
+      CLUTTER_NOTE (BACKEND, "Considering %li with type %d",
+                    info->id,
+                    info->use);
+
+      /* Only want 'raw' devices themselves not virtual ones */
+      if (info->use == IsXExtensionPointer ||
+        /*info->use == IsXExtensionKeyboard || XInput 1.x is broken */
+          info->use == IsXExtensionDevice)
+        {
+          clutter_x11_trap_x_errors ();
+          xdevice = XOpenDevice (backend_singleton->xdpy, info->id);
+          if (clutter_x11_untrap_x_errors () || xdevice == NULL)
+            continue;
+
+          /* Create the appropriate Clutter device */
+          device = g_slice_new0 (ClutterX11XInputDevice);
+
+          device->device.id = info->id;
+
+          switch (info->use)
+            {
+            case IsXExtensionPointer:
+              device->device.device_type = CLUTTER_POINTER_DEVICE;
+              have_an_xpointer = TRUE;
+              break;
+
+#if 0
+            /* XInput 1.x is broken for keyboards: */
+            case IsXExtensionKeyboard:
+              device->device.type = CLUTTER_KEYBOARD_DEVICE;
+              break;
+#endif
+
+            case IsXExtensionDevice:
+              device->device.device_type = CLUTTER_EXTENSION_DEVICE;
+              break;
+            }
+
+          /* FIXME: some kind of general device_init() call should do below */
+          device->device.click_count = 0;
+          device->device.previous_time = 0;
+          device->device.previous_x = -1;
+          device->device.previous_y = -1;
+          device->device.previous_button_number = -1;
+
+          device->num_events = 0;
+          device->xdevice = xdevice;
+
+          CLUTTER_NOTE (BACKEND, "Registering XINPUT device with XID: %li",
+                        xdevice->device_id);
+
+          /* We must go through all the classes supported by this device and
+           * register the appropriate events we want. Each class only appears
+           * once. We need to store the types with the stage since they are
+           * created dynamically by the server. They are not device specific.
+           */
+          for (j = 0; j < xdevice->num_classes; j++)
+            {
+              xclass_info = xdevice->classes + j;
+
+              switch (xclass_info->input_class)
+                {
+#if 0
+                /* XInput 1.x is broken for keyboards: */
+                case KeyClass:
+                  DeviceKeyPress (xdevice,
+                                  x11b->event_types[CLUTTER_X11_XINPUT_KEY_PRESS_EVENT],
+                                  device->xevent_list [num_events]);
+                  num_events++;
+
+                  DeviceKeyRelease (xdevice,
+                                    x11b->event_types[CLUTTER_X11_XINPUT_KEY_RELEASE_EVENT],
+                                    device->xevent_list [num_events]);
+                  num_events++;
+                  break;
+#endif
+
+                case ButtonClass:
+                  DeviceButtonPress (xdevice,
+                                     x11b->event_types[CLUTTER_X11_XINPUT_BUTTON_PRESS_EVENT],
+                                     device->xevent_list [num_events]);
+                  num_events++;
+
+                  DeviceButtonRelease (xdevice,
+                                       x11b->event_types[CLUTTER_X11_XINPUT_BUTTON_RELEASE_EVENT],
+                                       device->xevent_list [num_events]);
+                  num_events++;
+                  break;
+
+                case ValuatorClass:
+                  DeviceMotionNotify (xdevice,
+                                      x11b->event_types[CLUTTER_X11_XINPUT_MOTION_NOTIFY_EVENT],
+                                      device->xevent_list [num_events]);
+                  num_events++;
+                  break;
+                }
+            }
+
+          if (info->use == IsXExtensionPointer && num_events > 0)
+            have_an_xpointer = TRUE;
+
+          device->num_events  = num_events;
+
+          input_devices = g_slist_prepend (input_devices, device);
+        }
+    }
+
+  XFree (xdevices);
+
+  if (!have_an_xpointer)
+    {
+      GSList *l;
+
+      /* Something is likely wrong with Xinput setup so we basically
+       * abort here and fall back to lofi regular xinput.
+      */
+      g_warning ("No usuable XInput pointing devices found");
+
+      for (l = input_devices; l != NULL; l = l->next)
+        g_slice_free (ClutterX11XInputDevice, l->data);
+
+      g_slist_free (input_devices);
+      context->input_devices = NULL;
 
       return;
     }
 
-  manager = clutter_device_manager_get_default ();
+  /* store the list of input devices */
+  context->input_devices = g_slist_reverse (input_devices);
 
-  for (l = clutter_device_manager_peek_devices (manager);
-       l != NULL;
-       l = l->next)
+  /* why yes, we are awesome */
+  backend_singleton->have_xinput = TRUE;
+#endif /* HAVE_XINPUT */
+}
+
+void
+_clutter_x11_unregister_xinput ()
+{
+
+}
+
+void
+_clutter_x11_select_events (Window xwin)
+{
+#ifdef HAVE_XINPUT
+  GSList *list_it;
+  ClutterX11XInputDevice *device = NULL;
+
+  ClutterMainContext  *context;
+
+  context = _clutter_context_get_default ();
+
+  if (!backend_singleton)
+    {
+      g_critical ("X11 backend has not been initialised");
+      return;
+    }
+
+  for (list_it = context->input_devices;
+       list_it != NULL;
+       list_it = list_it->next)
   {
-    ClutterInputDevice *device = l->data;
+    device = (ClutterX11XInputDevice *)list_it->data;
 
-    _clutter_input_device_x11_select_events (device, backend_singleton, xwin);
+    XSelectExtensionEvent (backend_singleton->xdpy,
+                           xwin,
+                           device->xevent_list,
+                           device->num_events);
   }
 #endif /* HAVE_XINPUT */
 }
@@ -695,34 +876,49 @@ _clutter_x11_select_events (Window xwin)
 ClutterInputDevice *
 _clutter_x11_get_device_for_xid (XID id)
 {
-  ClutterDeviceManager *manager;
+#ifdef HAVE_XINPUT
+  ClutterMainContext *context;
+  GSList *l;
 
-  manager = clutter_device_manager_get_default ();
+  context = _clutter_context_get_default ();
 
-  return clutter_device_manager_get_device (manager, (gint) id);
+  if (!backend_singleton)
+    {
+      g_critical ("X11 backend has not been initialised");
+      return NULL;
+    }
+
+  for (l = context->input_devices; l != NULL; l = l->next)
+    {
+      ClutterX11XInputDevice *device = l->data;
+
+      if (device->xdevice->device_id == id)
+        return (ClutterInputDevice *) device;
+    }
+#endif /* HAVE_XINPUT */
+
+  return NULL;
 }
 
-/**
- * clutter_x11_get_input_devices:
- *
- * Retrieves a pointer to the list of input devices
- *
- * Deprecated: 1.2: Use clutter_device_manager_peek_devices() instead
- *
- * Since: 0.8
- *
- * Return value: a pointer to the internal list of input devices; the
- *   returned list is owned by Clutter and should not be modified or
- *   freed
- */
+/* FIXME: This nasty little func needs moving elsewhere.. */
 G_CONST_RETURN GSList *
 clutter_x11_get_input_devices (void)
 {
-  ClutterDeviceManager *manager;
+#ifdef HAVE_XINPUT
+  ClutterMainContext  *context;
 
-  manager = clutter_device_manager_get_default ();
+  if (!backend_singleton)
+    {
+      g_critical ("X11 backend has not been initialised");
+      return NULL;
+    }
 
-  return clutter_device_manager_peek_devices (manager);
+  context = _clutter_context_get_default ();
+
+  return context->input_devices;
+#else /* !HAVE_XINPUT */
+  return NULL;
+#endif /* HAVE_XINPUT */
 }
 
 /**
@@ -739,7 +935,7 @@ gboolean
 clutter_x11_has_xinput (void)
 {
 #ifdef HAVE_XINPUT
-  if (backend_singleton == NULL)
+  if (!backend_singleton)
     {
       g_critical ("X11 backend has not been initialised");
       return FALSE;
@@ -751,14 +947,6 @@ clutter_x11_has_xinput (void)
 #endif
 }
 
-/**
- * clutter_x11_has_composite_extension:
- *
- * Retrieves whether Clutter is running on an X11 server with the
- * XComposite extension
- *
- * Return value: %TRUE if the XComposite extension is available
- */
 gboolean
 clutter_x11_has_composite_extension (void)
 {
@@ -792,58 +980,9 @@ clutter_x11_has_composite_extension (void)
   return have_composite;
 }
 
-/**
- * clutter_x11_set_use_argb_visual:
- * @use_argb: %TRUE if ARGB visuals should be requested by default
- *
- * Sets whether the Clutter X11 backend should request ARGB visuals by default
- * or not.
- *
- * By default, Clutter requests RGB visuals.
- *
- * <note>If no ARGB visuals are found, the X11 backend will fall back to
- * requesting a RGB visual instead.</note>
- *
- * ARGB visuals are required for the #ClutterStage:use-alpha property to work.
- *
- * <note>This function can only be called once, and before clutter_init() is
- * called.</note>
- *
- * Since: 1.2
- */
-void
-clutter_x11_set_use_argb_visual (gboolean use_argb)
-{
-  if (backend_singleton != NULL)
-    {
-      g_warning ("%s() can only be used before calling clutter_init()",
-                 G_STRFUNC);
-      return;
-    }
-
-  CLUTTER_NOTE (BACKEND, "ARGB visuals are %s",
-                use_argb ? "enabled" : "disabled");
-
-  clutter_enable_argb = use_argb;
-}
-
-/**
- * clutter_x11_get_use_argb_visual:
- *
- * Retrieves whether the Clutter X11 backend is using ARGB visuals by default
- *
- * Return value: %TRUE if ARGB visuals are queried by default
- *
- * Since: 1.2
- */
-gboolean
-clutter_x11_get_use_argb_visual (void)
-{
-  return clutter_enable_argb;
-}
-
 XVisualInfo *
-clutter_backend_x11_get_visual_info (ClutterBackendX11 *backend_x11)
+clutter_backend_x11_get_visual_info (ClutterBackendX11 *backend_x11,
+                                     gboolean           for_offscreen)
 {
   ClutterBackendX11Class *klass;
 
@@ -851,29 +990,7 @@ clutter_backend_x11_get_visual_info (ClutterBackendX11 *backend_x11)
 
   klass = CLUTTER_BACKEND_X11_GET_CLASS (backend_x11);
   if (klass->get_visual_info)
-    return klass->get_visual_info (backend_x11);
+    return klass->get_visual_info (backend_x11, for_offscreen);
 
   return NULL;
 }
-
-/**
- * clutter_x11_get_visual_info:
- *
- * Retrieves the <structname>XVisualInfo</structname> used by the Clutter X11
- * backend.
- *
- * Return value: a <structname>XVisualInfo</structname>, or
- *   <varname>None</varname>. The returned value should be freed using XFree()
- *   when done
- *
- * Since: 1.2
- */
-XVisualInfo *
-clutter_x11_get_visual_info (void)
-{
-  ClutterBackendX11 *backend_x11;
-
-  backend_x11 = CLUTTER_BACKEND_X11 (clutter_get_default_backend ());
-
-  return clutter_backend_x11_get_visual_info (backend_x11);
-}
