@@ -14,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef __CLUTTER_STAGE_X11_H__
@@ -50,16 +50,21 @@ struct _ClutterStageX11
 {
   ClutterGroup parent_instance;
 
-  guint        is_foreign_xwin      : 1;
-  guint        fullscreening        : 1;
-  guint        is_cursor_visible    : 1;
-  guint        viewport_initialized : 1;
+  guint        is_foreign_xwin    : 1;
+  guint        fullscreen_on_map  : 1;
+  guint        is_cursor_visible  : 1;
 
-  Window       xwin;
+  Display     *xdpy;
+  Window       xwin_root;
+  int          xscreen;
+  XVisualInfo *xvisinfo;
+  Window       xwin;  
   gint         xwin_width;
   gint         xwin_height; /* FIXME target_width / height */
+  Pixmap       xpixmap;
   gchar       *title;
 
+  ClutterBackendX11 *backend;
   ClutterStageState  state;
 
   ClutterStageX11State wm_state;
