@@ -41,7 +41,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
-#include <clutter/clutter-stage.h>
+#include <clutter/clutter.h>
 #include <clutter/x11/clutter-x11-enum-types.h>
 #include <clutter/x11/clutter-x11-texture-pixmap.h>
 
@@ -74,12 +74,6 @@ typedef enum {
   CLUTTER_X11_XINPUT_LAST_EVENT
 } ClutterX11XInputEventTypes;
 
-typedef enum {
-  CLUTTER_X11_XINPUT_POINTER_DEVICE,
-  CLUTTER_X11_XINPUT_KEYBOARD_DEVICE,
-  CLUTTER_X11_XINPUT_EXTENSION_DEVICE
-} ClutterX11InputDeviceType;
-
 typedef struct _ClutterX11XInputDevice ClutterX11XInputDevice;
 
 /**
@@ -118,26 +112,19 @@ void         clutter_x11_remove_filter (ClutterX11FilterFunc func,
 
 ClutterX11FilterReturn clutter_x11_handle_event (XEvent *xevent);
 
-void         clutter_x11_disable_event_retrieval (void);
-
-gboolean     clutter_x11_has_event_retrieval (void);
+void     clutter_x11_disable_event_retrieval (void);
+gboolean clutter_x11_has_event_retrieval (void);
 
 ClutterStage *clutter_x11_get_stage_from_window (Window win);
 
-GSList*
-clutter_x11_get_input_devices (void);
+G_CONST_RETURN GSList* clutter_x11_get_input_devices (void);
 
-ClutterX11InputDeviceType
-clutter_x11_get_input_device_type (ClutterX11XInputDevice *device);
+void     clutter_x11_enable_xinput (void);
+gboolean clutter_x11_has_xinput (void);
 
-void
-clutter_x11_enable_xinput ();
+gboolean clutter_x11_has_composite_extension (void);
 
-gboolean
-clutter_x11_has_xinput (void);
-
-gboolean
-clutter_x11_has_composite_extension (void);
+Time clutter_x11_get_current_event_time (void);
 
 G_END_DECLS
 
